
#include <NewPing.h>    //Updated 2015 library for ultrasound sensor
//#include <PololuWheelEncoders.h>

#define TRIGPIN A6      //What is the digital pin number?
#define ECHOPIN A7      //What is the digital pin number?
#define MAXDIST 400     //sets maximum useable sensor measuring distance to 300cm
#define COLLDIST 15     //motors are faster so set distance further out
#define ALERTDIST 30
NewPing sonar(TRIGPIN, ECHOPIN, MAXDIST);  // sets up sensor library to use the correct pins to measure distance.

// TB6612FNG and 1284P pins
#define PWMA 13
#define AIN1 18
#define AIN2 19
#define PWMB 14
#define BIN1 3
#define BIN2 2
#define STBY 12

// Motor speed defines
#define MAXSPEED 240
#define SPEEDFACT 0.8
//#define OFFSET -10    // In lieu of potentiometer for adjusting speed offset


#define LEFTIR A5    //Port A, PCINT 5
#define RIGHTIR A4   //Port A, PCINT 4


int d = 0; // turn delay variable
int curDist = 0;

//***************** Smoothing *****************
//*********************************************
//From: http://arduino.cc/en/Tutorial/Smoothing
const int numReadings = 5;

int readings[numReadings];      // the readings from the analog input
int index = 0;                  // the index of the current reading
int total = 0;                  // the running total
int average = 0;                // the average
//*********************************************


//**************************** SETUP ****************************************
//***************************************************************************
void setup() {

  // init encoders-m2a & m2b pins are swapped due to rotation of motors, gives positive counts
  // when moving in forward direction
  //PololuWheelEncoders::init(22,23,21,20);   // 1284P-PU //(m1a, m1b, m2a, m2b)

  //Serial.begin(115200);
  pinMode(LEFTIR, INPUT);
  pinMode(RIGHTIR, INPUT);
  //enable pin change interrupts, mask for specific pins
  PCICR |= (1<<PCIE0); //enable group interrupts on PORTA PCINT[7:0]
  PCMSK0 |= (1<<PCINT5); //mask to enable interrupt on A5 for LEFTIR
  PCMSK0 |= (1<<PCINT4); //mask to enable interrupt on A4 for RIGHTIR
  
  pinMode(PWMA,OUTPUT);
  pinMode(AIN1,OUTPUT);
  pinMode(AIN2,OUTPUT);
  pinMode(PWMB,OUTPUT);
  pinMode(BIN1,OUTPUT);
  pinMode(BIN2,OUTPUT);
  pinMode(STBY,OUTPUT);

  for (int thisReading = 0; thisReading < numReadings; thisReading++)
    readings[thisReading] = 0;   // smoothing funxn

  startUp();
  delay(2000);
}

//************************ LOOP *********************************************
//***************************************************************************
void loop() {
  curDist = distAverage();     //sonar.ping_cm();   // read distance
  if (curDist > MAXDIST)
  {
    curDist == MAXDIST;
  }
  if (curDist < ALERTDIST)
  {
    if (curDist < COLLDIST)
    {
      stopAll();
      Backward();
      delay(100);
      newPath();
    }
    else
    {
      newPath();
    }
  }  
  else Forward();
  // gyro and accel funxns for collision detect
  
  //checkLR();    // moving to pin change interrupts for this, rather than constantly polling sensors
}     

//*************** PIN CHANGE INTERRUPTS ISR FOR SHARP IRs ******************
//**************************************************************************
ISR (PCINT0_vect) //handle pin change interrupt for group A0-A7, triggered by A5 & A4
{
  if ( bitRead (PINA,5) == LOW)
  {
    veerRight();
  }
  if ( bitRead (PINA,4) == LOW)
  {
    veerLeft();
  }
  if ( bitRead (PINA,5) == LOW && bitRead (PINA,4) == LOW)
  {
    backUp();
  }
}

//************************ FUNCTIONS ****************************************
//***************************************************************************

int distAverage(){
  total= total - readings[index];      // subtract the last reading 
  readings[index] = sonar.ping_cm();   //analogRead(inputPin); // read from the sensor
  total= total + readings[index];      // add the reading to the total
  index = index + 1;                   // advance to the next position in the array               

  if (index >= numReadings)            // if we're at the end of the array...         

    index = 0;                         // ...wrap around to the beginning                      

  average = total / numReadings;         
  delay(1);                            // delay in between reads for stability
  return average;  
}

void newPath()
{
  stopAll();

  if (random(2) == 0)
  {
    rotateLeft();
    d = ((random(4) + 1) * 100);
    delay(d);
    senseLeft();
  }
  else rotateRight();
  d = ((random(4) + 1) * 100);
  delay(d);
  senseRight();
}

void senseLeft()
{
  curDist = sonar.ping_cm();   // read distance
  if ( curDist < COLLDIST )
  {
    rotateLeft();
    delay(200);
  }
  else Forward();
}

void senseRight()
{
  curDist = sonar.ping_cm();   // read distance
  if ( curDist < COLLDIST )
  {
    rotateRight();
    delay(200);
  }
  else Forward();
}
/*
void checkLR()         // Polled for in the main loop funxn
{
  int leftVal = digitalRead(LEFTIR);  //SharpIR, A5
  int rightVal = digitalRead(RIGHTIR);  //Sharp IR, A4
  if (rightVal == LOW && leftVal == LOW)
  {
    stopAll();
    Backward();
    delay(400);
    stopAll();
    delay(100);
    newPath();
  }
  if (leftVal == LOW)
  {
    stopAll();
    rotateRight();
    delay(100);
  } 
  if (rightVal == LOW)
  {
    stopAll();
    rotateLeft();
    delay(100);
  }
}
*/

//**************************** MOTOR CONTROL ********************************
//***************************************************************************
void Forward ()
{
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,HIGH);
  digitalWrite (AIN2,LOW);
  analogWrite(PWMA,MAXSPEED + potValue);    // Offset for speed diff of motors
  digitalWrite (BIN1,HIGH);
  digitalWrite (BIN2,LOW);
  analogWrite(PWMB,MAXSPEED);  
}
void Backward ()
{
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,LOW);
  digitalWrite (AIN2,HIGH);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,LOW);
  digitalWrite (BIN2,HIGH);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);  
}
void stopAll ()
{
  digitalWrite (AIN1,HIGH);
  digitalWrite (AIN2,HIGH);
  analogWrite(PWMA,255);
  digitalWrite (BIN1,HIGH);
  digitalWrite (BIN2,HIGH);
  analogWrite(PWMB,255);  
}
void rotateLeft ()
{
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,HIGH);
  digitalWrite (AIN2,LOW);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,LOW);
  digitalWrite (BIN2,HIGH);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);  
}
void rotateRight ()
{
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,LOW);
  digitalWrite (AIN2,HIGH);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,HIGH);
  digitalWrite (BIN2,LOW);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);  
}

void veerRight(){
  /*
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,LOW);
  digitalWrite (AIN2,HIGH);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,HIGH);
  digitalWrite (BIN2,LOW);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);
  */                    // commented out above for calling rotateRight()
  rotateRight();        // long delay to see if something happens
  delay(2000);          // result is tiny course change, but not the 2000ms duration
}

void veerLeft(){
  /*
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,HIGH);
  digitalWrite (AIN2,LOW);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,LOW);
  digitalWrite (BIN2,HIGH);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);
  */
  rotateLeft();
  delay(2000);
}

void backUp(){
  int potValue = analogRead(A3);
  potValue = map(potValue, 0, 1023, -15, +15);
  digitalWrite (AIN1,LOW);
  digitalWrite (AIN2,HIGH);
  analogWrite(PWMA,(MAXSPEED + potValue)*SPEEDFACT);
  digitalWrite (BIN1,LOW);
  digitalWrite (BIN2,HIGH);
  analogWrite(PWMB,MAXSPEED*SPEEDFACT);

  delay(2000);       //long delay to see if something happens...and nothing happens
}

void startUp ()
{
  digitalWrite(STBY,HIGH);
}
void shutDown ()
{
  digitalWrite(STBY,LOW);
}


